#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2016 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end


require 'optparse'
require 'optparse/time'
require 'ostruct'
require_relative 'logging'
require_relative 'translating'
require_relative 'constants'

class ArgParser 

  self.extend(Logging)
  self.extend(Translating)
  @@log = init_logger()
  
  # Returns a structure describing the options.
  #
  def self.parse(args)
    # The options specified on the command line will be collected in <b>options</b>.
    # We set default values here.

    options = OpenStruct.new
    options.debug = false 
    options.table_name = false

    op = OptionParser.new do |opts|
      opts.banner = "\n" << trl("Usage") << ":\t" << $APPNAME.dup << ' ' << trl("-s [SQLite-file] [options]") << "\n\t" << trl("or %s [Common options]") %($APPNAME) 

      opts.separator ""
      opts.separator trl("Specific options") << ':'

      opts.on('-' << trl("s"), '--' << trl("source [PATH]"), trl( "SQLite-file to read.")) do |source|
	options.source = source
      end

      opts.on('-' << trl('c'), '--' << trl("config [PATH]"), trl('Configuration file for this transformation')) do |config|
              options.config = config
      end

      opts.on('-' << trl('n'), '--' << trl("name [TABLE]"), trl('The name of the table from the SQLite-database to convert')) do |name|
	options.name = name
      end

      opts.on('-' << trl("t"), '--' << trl("target [PATH]"), trl( "Path to the dBase-file to be written.")) do |target|
	options.target = target
      end

      opts.on('-' << trl('o'), '--' << trl("out [PATH]"), trl('Use the table-name as file-name for the DBF-result, store output in PATH')) do |path|
	options.name_like_table = true
	options.out = path
      end
      opts.on('--' << trl("time [list]"), trl( "Fields (table-columns) which shall be handled as timestamp values.")) do |list|
	options.time = list.gsub(/[,;]/, '').split
      end

      opts.on('--' << trl("date [list]"), trl( "Fields (table-columns) which shall be handled as date-time values.")) do |list|
	options.datetime = list.gsub(/[,;]/, '').split
      end

=begin

# This is for later. The evaluation of the file-content may be allocated to the
# Configuration singleton, as it does not mean a lot of I/O nor does it imply 
# much management.

opts.on("-c", "--config [CONFIG FILE]",
	"Read alternative configuration from this file") do |conf|
	$LOG.debug("config-file should be #{conf}") if $LOG
	options.config = File.expand_path(conf )
end
opts.on("-g", "--generic [CONFIG FILE]",
	"Write generic configuration options to this file") do |file|
	options.generic = File.expand_path(file )
end
=end
      opts.separator ""
      opts.separator trl("Common options") << ':'

      # No argument, shows at tail.  This will print an options summary.
      #
      opts.on_tail(trl("-d"), trl("--debug"), trl("Show debug-messages") ) do
	options.debug = true
	@@log.level = Logger::DEBUG
      end


      opts.on_tail(trl("-h"), trl("--help"), trl("Show this message") ) do
	puts opts
	exit true
      end

      opts.on_tail(trl("-v"), trl("--version"), trl("Show version and program information") ) do
	puts "\t#{$APPNAME}"
	puts "\t© #{$YEARS}, } <#{$AUTHORS.join(', ')}>"
	exit true
      end
    end
    begin
      op.parse!(args)
    rescue OptionParser::ParseError => er
      msg = trl("ERROR! Unsuitable or incomplete program-arguments") << (": %s" %er.message	)
      puts msg
      puts trl("Start this program with parameter -h or --help to see the usage-message.")
      exit false
    end

    options
  end  # parse()

end  

