#!/usr/bin/env ruby
#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2016 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end


require_relative 'file_checking'
require_relative 'logging'
require_relative 'configuration'
require 'shp'
require 'sqlite3'
require_relative 'argparser'

=begin
        The main program class. Does it.
=end

class SQLite2DBF
  include Logging
  include Translating

  def initialize(*args)
    options = ArgParser::parse(args)
    @config = Configuration.instance()
    @config.set(options)

    init_logger
    level = (@config.debug ? Logger::DEBUG : @log.level)
    @log.level = level	

    @date_fields = @config.datetime ? @config.datetime : []
    @time_fields = @config.time ? @config.time : []

    if(@config.source)
      @dbf_path = nil
      sqlite_file = @config.source	
      dbf_file = @config.target if @config.target
      if(@config.name_like_table && @config.out_dir)
        @dbf_path = @config.out_dir
        dbf_file = :table
      end
      dbf_file ||= File.dirname(sqlite_file) << File::Separator << File.basename(sqlite_file, '.*')
      msg = nil

      if(dbf_file != :table)
        if(File.exist?(dbf_file))
          msg = File_Checking.file_check(dbf_file, :file, :writable)
        elsif(File.exist?(File.dirname(dbf_file))) 
          msg = File_Checking.file_check(File.dirname(dbf_file), :directory, :writable)
        end
      end

      msg ||= File_Checking.file_check(sqlite_file, :exist, :readable, :file)

      if(!msg)
        @log.debug('will transform ' << sqlite_file << ' to  ' << dbf_file.to_s << '.dbf')
        transform(sqlite_file, dbf_file)
      else
        msg = trl("ERROR! Unsuitable file") << " : %s" %msg
        @log.error(msg)
        exit false
      end
    else
      puts trl("ERROR! Source-file is a mandatory program parameter!")
      puts trl("Start this program with parameter -h or --help to see the usage-message.")  
    end

  end

  private

  def transform(sqlite_file, dbf_file)
    # <------------ mapping SQLite-types to SHP/XBase ------------>
    tmap = {'INT' => 1, 'TINYINT' => 1, 'SMALLINT' => 1, 'MEDIUMINT' => 1, 'BIGINT' => 1, 'UNSIGNED BIGINT' => 1, 'VARCHAR' => 0, 'CHARACTER' => 0, 'VARYING CHARACTER' => 0, 'LONGVARCHAR' => 0, 'TEXT' => 0, 'BLOB' => 0, 'INTEGER' => 1, 'FLOAT' => 2, 'REAL' => 2, 'DOUBLE' => 2}
    # -----------> in a way ... <--------------

    begin
      SQLite3::Database.new(sqlite_file) do |db|
        table_names = db.execute('select name from sqlite_master WHERE type="table"') 
        @log.debug('sqlite contains ' << table_names.size.to_s << ' table(s): ' << table_names.join(', '))
        table_names.each_with_index do |tn, num| 
          tname = tn.join
          table_info = db.table_info(tname) 
          @log.debug(tn[0] << ': ' << table_info.join)
          content = db.execute('select * from ' << tname ) 
          if(:table == dbf_file)	
            dbf_name = @dbf_path << File::Separator << tname << (content.empty? ? '_empty' : '')<< '.dbf'
          else
            dbf_name = dbf_file.dup << num.to_s << (content.empty? ? '_empty' : '')<< '.dbf'
          end
          @log.debug('dbf will be ' << dbf_name)
          dbt = SHP::DBF.create(dbf_name) 
          table_info.each_with_index do |field, index|
            ftype = 0 
            field_name = field['name']
            if( !@date_fields.include?(field_name) && !@time_fields.include?(field_name) )
              @log.debug('field is ' << field.to_s)
              # -----> determine the simple dbf data-type that has to do. <----
              type_mapping = tmap.detect{|key, value| field['type'].upcase.start_with?(key) }
              @log.debug('type_mapping is ' << type_mapping.to_s)
              ftype = type_mapping[1] if type_mapping && type_mapping.respond_to?(:to_ary)
              # <---- Yes. I know. ---->
            end

            dbt.add_field(field_name, ftype, 200, 0)
          end

          content.each_with_index do |row, record_no|
            @log.debug('row is ' << row.to_s)
            row.each_with_index do |svalue, field_index|
              sql_type = table_info[field_index]['type']
              field_name = table_info[field_index]['name']
              @log.debug('field is ' << field_name << ', svalue is ' << svalue.to_s << ', type is ' << sql_type)
              if(svalue && !svalue.to_s.empty?)
                if( @date_fields.include?(field_name) )
                  conv_date = db.execute("select strftime('%Y-%m-%d %H:%M:%S', " << svalue.to_s << ", 'unixepoch')").join
                  @log.debug('conv_date is ' << conv_date)
                  dbt.write_string_attribute(record_no, field_index, conv_date)
                elsif( @time_fields.include?(field_name) )
                  conv_time = db.execute("select strftime('%s', " << svalue.to_s << ", 'unixepoch')").join
                  @log.debug('conv_time is ' << conv_time)
                  dbt.write_string_attribute(record_no, field_index, conv_time)

                else
                  type_mapping = tmap.detect{|key, value| sql_type.upcase.start_with?(key) }
                  ftype = type_mapping[1] if type_mapping && type_mapping.respond_to?(:to_ary)
                  begin
                    dbt.write_string_attribute(record_no, field_index, svalue ) if 0 == ftype
                    dbt.write_integer_attribute(record_no, field_index, svalue) if 1 == ftype
                    dbt.write_double_attribute(record_no, field_index, svalue) if 2 == ftype
                  rescue StandardError => ex
                    @log.error(trl("ERROR! cannot write value of type %s" %sql_type) << ': ' << ex.message )
                  end
                end
              else
                dbt.write_null_attribute(record_no, field_index) if (!svalue || sql_type.empty?)
              end
            end
          end
          dbt.close
        end
      end 
      puts trl('DONE. Bye.')
    end
  rescue SQLite3::Exception => er
    @log.error trl("ERROR! Cannot read the source-file") << ' (' << sqlite_file << "): %s" %er.message << '.'
  end
end
